
//=============================================
// 
// Mercredi 12 decembre 2001
//  
// Arash Habibi
// 
// Vecteur.c
//
//============================================


#include <Vecteur.h>


//-------------------------------------------
// Affiche a l'ecran les composantes. 
//-------------------------------------------

void V_Check(vecteur_t v, char *commentaire)
{
  fprintf(stderr,"%s : \t%f\t%f\t%f\n", commentaire, v.x, v.y, v.z);
}


//-------------------------------------------
// renvoie 1 si v est nul et 0 sinon. 
//-------------------------------------------

int V_EstNul(vecteur_t v)
{
  return((v.x==0.0)&&(v.y==0.0)&&(v.z==0.0)); 
}

//-------------------------------------------
// Affecte les valeurs x, y et z au vecteur v. 
//-------------------------------------------

vecteur_t V_Set(vecteur_t *v, double x, double y, double z) 
{
  v->x = x; 
  v->y = y; 
  v->z = z; 
  return *v;
}

//-------------------------------------------
// retourne la somme de v1 et de v2. 
//-------------------------------------------

vecteur_t V_Addition2(vecteur_t v1, vecteur_t v2) 
{
  vecteur_t somme; 

  somme.x = v1.x + v2.x; 
  somme.y = v1.y + v2.y; 
  somme.z = v1.z + v2.z; 

  return(somme);
}

//-------------------------------------------
// retourne la somme de v1, v2 et de v3. 
//-------------------------------------------

vecteur_t V_Addition3(vecteur_t v1, vecteur_t v2, vecteur_t v3) 
{
  vecteur_t somme; 

  somme.x = v1.x + v2.x + v3.x; 
  somme.y = v1.y + v2.y + v3.y; 
  somme.z = v1.z + v2.z + v3.z; 

  return(somme);
}

//-------------------------------------------
// retourne la somme de v1, v2, v3 et de v4. 
//-------------------------------------------

vecteur_t V_Addition4(vecteur_t v1, vecteur_t v2, vecteur_t v3, vecteur_t v4) 
{
  vecteur_t somme; 

  somme.x = v1.x + v2.x + v3.x + v4.x; 
  somme.y = v1.y + v2.y + v3.y + v4.y; 
  somme.z = v1.z + v2.z + v3.z + v4.z; 

  return(somme);
}

//-------------------------------------------
// retourne la difference v1-v2. 
//-------------------------------------------

vecteur_t V_Difference(vecteur_t v1, vecteur_t v2) 
{
  vecteur_t difference; 

  difference.x = v1.x - v2.x; 
  difference.y = v1.y - v2.y; 
  difference.z = v1.z - v2.z; 

  return(difference);
}

//-------------------------------------------
// retourne le produit du vecteur par le scalaire. 
//-------------------------------------------

vecteur_t V_Homothetie(double lambda, vecteur_t v) 
{
  vecteur_t produit; 

  produit.x = lambda * v.x; 
  produit.y = lambda * v.y; 
  produit.z = lambda * v.z; 

  return(produit); 
}

//-------------------------------------------
// axe vaut V_X, V_Y ou V_Z
//-------------------------------------------

vecteur_t V_ProjectionOrtho(vecteur_t v, int axe)
{
  vecteur_t projete; 
  
  projete = v; 

  if(axe==V_X)
    projete.x=0.0; 
    
  else if(axe==V_Y)
    projete.y=0.0; 
    
  else // if(axe==V_Z)
    projete.z=0.0; 
    
  return(projete);
}

//-------------------------------------------
// retourne le produit scalaire de v1 et de
// v2. 
//-------------------------------------------

inline double V_ProduitScalaire(vecteur_t v1, vecteur_t v2) 
{
  return(v1.x * v2.x + v1.y * v2.y + v1.z * v2.z);
}

//-------------------------------------------
// retourne la norme euclidienne de v. 
//-------------------------------------------

double V_Norme(vecteur_t v) 
{
  return(sqrt(V_ProduitScalaire(v, v)));
}

//-------------------------------------------
// retourne la distance entre les deux points
// v1 et v2, ce qui revient a retourner la 
// norme de la difference v1-v2. 
//-------------------------------------------

double V_Distance(vecteur_t v1, vecteur_t v2) 
{
  return(V_Norme(V_Difference(v1,v2))); 
}

//-------------------------------------------
// retourne un vecteur colineaire a v mais 
// de norme 1. Si v est nul, alors la direction
// du resultat sera arbitraire (en l'occurrence
// l'axe des x). 
//-------------------------------------------

vecteur_t V_Unitaire(vecteur_t v) 
{
  double norme; 
  vecteur_t axe_x; 

  norme = V_Norme(v); 
  if(norme==0.0)
    {
      V_Set(&axe_x, 1.0, 0.0, 0.0);
      return(axe_x);
    }
  else
    return(V_Homothetie(1.0/norme, v)); 
}

//-------------------------------------------
// retourne le produit vectoriel de v1 et de v2. 
//-------------------------------------------

vecteur_t V_ProduitVectoriel(vecteur_t v1, vecteur_t v2) 
{
  vecteur_t produit; 

  produit.x = v1.y * v2.z - v1.z * v2.y; 
  produit.y = v1.z * v2.x - v1.x * v2.z; 
  produit.z = v1.x * v2.y - v1.y * v2.x; 

  return(produit); 
}


//-------------------------------------------
// retourne la distance entre le point M et 
// la droite (A,u). Principe : 
// AM = AH + HM donc HM = AM - AH
// or c'est ||HM|| qu'on cherche. 
// et AH = (AM.u) u donc le resultat final 
// est || AM - (AM.u) u ||
//-------------------------------------------

double V_DistPointDroite(vecteur_t M, vecteur_t A, vecteur_t u)
{
  vecteur_t AM, unit_u; 

  unit_u = V_Unitaire(u); 
  AM = V_Difference(M,A); 

  return(V_Norme(V_Difference(AM, V_Homothetie(V_ProduitScalaire(AM,unit_u), unit_u))));
}
